/*********************************************************************/
/* BUSCAMINAS                                                        */
/*                                                                   */
/* Realizado por:                                                    */
/*               - Alejandro Muoz Andrade                           */
/*                                                                   */
/* Para compilar este programa utilice la opcion File->Change dir... */
/* y seleccione la carpeta donde se encuentran todos los archivos.   */
/*********************************************************************/
/* ACTUALIZACIONES:                                                  */
/*                                                                   */
/* - Implementada la presentacion.                                   */
/* - Implementada la funcion iniciar_svga que inicia el modo grafico */
/*   con el driver SVGA256, y junto con ella la funcion func, que    */
/*   establece el tamao en pixeles de la pantala.(hasta 1024x768)   */
/* - Implementada la funcion reiniciar_puntuaciones.                 */
/* - Implementada la funcion mostrarcuadro, que muestra la posicion  */
/*   actual de juego.                                                */
/* - Basicamente se hicieron cambios y mejoras a todas las funciones,*/
/*   pero a continuacion se muestran los mas destacados.             */
/* - Mejorada la funcion mensaje, ahora calcula la longitud del texto*/
/*   que aparecera y calcula el tamao que tendra el boton, se le    */
/*   deben pasar como parametros las coordenadas en pixeles del      */
/*   centro del boton y el texto a mostrar.                          */
/* - Mejorada la funcion guardar_puntuaciones, las puntuaciones se   */
/*   leen en un arreglo y se utiliza el metodo de la burbuja para    */
/*   ordenar los puntajes de menor a mayor.                          */
/* - Mejoradas la funcion ponerminas y sinmina, se cambiaron las     */
/*   condiciones if-else anidadas por un switch.                     */
/* - Mejoradas las funciones de manejo de archivos, ya no se cuelga  */
/*   el programa cuando no encuentra el archivo de puntajes.         */
/* - Mejorada la reserva de memoria dinamica para mostrar cuadros de */
/*   dialogo, ya no se cuelga el programa cuando no hay suficiente   */
/*   memoria, se imprime un mensaje de error de memoria.             */
/* - Se renombro la funcion ponernumeros a mostrarnumeros.           */
/* - Ampliada la biblioteca TECLAS.H con las definiciones para las   */
/*   teclas mayusculas. Todavia no se modifica el juego para que las */
/*   acepte.                                                         */
/* - Los numeros que representan los colores en cada funcion grafica,*/
/*   se cambiaron por los nombres respectivos.                       */
/* - Se incluye la libreria PCXBGI1.h para mostrar imagenes PCX, de  */
/*   cualquier resolucion y de 256 colores, utilizando para esto el  */
/*   drive SVGA256.BGI que fue descargado de Internet junto con la   */
/*   libreria (Solo usados en la presentacion).                      */
/*********************************************************************/

#include<dos.h>
#include<conio.h>
#include<ctype.h>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<graphics.h>

/****************************************************/
/* Libreria para el uso del teclado en codigo ASCII */
/****************************************************/
#include"teclas.h"
/****************************************************/
/* Libreria para mostrar imagenes PCX               */
/****************************************************/
#include"pcxbgi1.h"

/***********************************************************/
/* Definicion de la mitad de la pantalla y del maximo de   */
/* pantalla en modo grafico:                               */
/*   maxx -> Maximo Horizontal                             */
/*   maxy -> Maximo Vertical                               */
/*   mitx -> Mitad Horizontal                              */
/*   mity -> Mitad Vertical                                */
/***********************************************************/
#define maxx 639
#define maxy 479
#define mitx 319
#define mity 239

/***********************************************************/
/* Definicion del tamao de las filas y columnas de la     */
/* matriz.                                                 */
/*   tamx -> Maximo Horizontal (columnas)                  */
/*   tamy -> Maximo Vertical (filas)                       */
/***********************************************************/
#define tamx 22
#define tamy 16

/*****************************************************/
/* Definicion del numero de minas para los 3 Niveles */
/*   Nivel 1 -> 10 Minas                             */
/*   Nivel 2 -> 30 Minas                             */
/*   Nivel 3 -> 50 Minas                             */
/*****************************************************/
#define minas1        10
#define minas2        30
#define minas3        50

/***************************************************************************/
/* Declaracion de Variables Globales                                       */
/*                                                                         */
/* xp: Posicion X en la matrix tablero2(Columnas)                          */
/* yp: Posicion Y en la matriz tablero2(Filas)                             */
/* xg y yg: Posiciones X y Y para trabajar en modo grafico, realizando las */
/*          respectivas conversiones de xp y yp de la matriz.              */
/* tablero2[tamx][tamy]: Matriz que representa el tablero de juego             */
/* xp_ant: Posicion anterior X del jugador en la matrix tablero_temp       */
/* yp_ant: Posicion anterior Y del jugador en la matriz tablero_temp       */
/* xg_ant y yg_ant: Posiciones X y Y anteriores del jugador para trabajar  */
/*                  en modo grafico, realizando las respectivas            */
/*                  conversiones de xp_ant y yp_ant de la matriz           */
/* minas: Numero de minas que tendra el tablero de juego(30 por defecto)   */
/* minas4: Numero de minas que se asignaran al nivel personalizado (no la  */
/*         defino como constante debido a que el numero de minas de este   */
/*         nivel estara dado por el usuario)                               */
/* decremento: Numero de espacios recorridos, se decrementa hasta que se   */
/*             sea igual al numero de minas.                               */
/* tablero_decremento: se usa para contar el numero de espacios que han    */
/*                     sido destapados.                                    */
/* puntos: Contador para el numero de puntos que va ganando el jugador     */
/*         (Cada vez que se mueva el jugador se incrementara en 100)       */
/* dec,sig: Variables utilizadas para convertir un numero a una cadena     */
/* opc1: Lee una opcion del teclado en la funcion principal(main), y es    */
/*       usada tambien para salir del juego en la funciones menu y salir   */
/* opc2: Lee una opcion del teclado en la funcion menu(), tambien usada en */
/*       la funcion menuniveles para continuar con el juego                */
/* opc3: Lee una opcion del teclado en la funcion ganador(), para iniciar  */
/*       un nuevo juego, tambien es usada en las funciones menu y pantalla */
/*       como opciones para continuar con el juego                         */
/* opc_niveles: Lee una opcion del teclado en la funcion menuniveles(),    */
/*              tambien usada en la funcion principal(main) y menu para    */
/*              comprobar que boton fue presionado                         */
/* opc_salida: Lee una opcion del teclado en la funcion salir(), para salir*/
/*             del juego(S  N)                                            */
/* *puntuacion: Guarda la puntuacion obtenida de la variable puntos como   */
/*              una cadena de caracteres                                   */
/* nombre_puntuacion: Guarda el nombre del usuario que obtuvo una buena    */
/*                    puntuacion                                           */
/* paleta: Guarda la informacion que contiene la paleta del sistema para la*/
/*         actual tarjeta grafica                                          */
int xp,yp,xg,yg,tablero2[tamx][tamy];
int xp_ant,yp_ant,xg_ant,yg_ant;
int minas=30;
int minas4=0;
int decremento;
int tablero_decremento[tamx][tamy];
float puntos=0;
int dec,sig;
char opc1,opc2,opc3,opc_niveles,opc_salida;
char *puntuacion,nombre_puntuacion[16];
struct palettetype paleta;

/********************************************************************/
/* Variables para guardar la puntuacion                             */
/* *fp: Puntero al archivo donde estan guardadas las puntuaciones   */
/* puntajes: Nuevo tipo de datos que representa una estructura para */
/*           guardar las puntuaciones                               */
/********************************************************************/
FILE *fp;

typedef struct strpuntajes
{
 char nombre[16];
 char puntaje[10];
}puntajes;

/*******************************/
/* Prototipos de las Funciones */
/*******************************/
void iniciar_graficos(void);
int huge func(void);
void iniciar_svga(void);
void presentacion(void);
void salir(void);
void pantalla(void);
void menu(void);
int  menuniveles(void);
void mostrar_puntuacion(void);
int  personalizado(void);
void carita(int xc,int yc,int cfondo);
void caritamala(int xc,int yc,int cfondo);
void ganador(void);
void reiniciar_puntuaciones(void);
void guardar_puntuacion(void);
void guardar_nombre(void);
void perdedor(void);
void ponerminas(int num);
void explotarminas(void);
void mostrarnumeros(char *num2,int colornum,int colorfondo);
void sinmina(void);
void sonido(int frecuencia,long int tiempo);
void mina(int xg,int yg);
void mostrarcuadro(void);
void mensaje(int x,int y,char *mens);
void acercade(void);
void ayuda(void);
void paginas(int num);

/**************************************/
/* Programa Principal -> "BUSCAMINAS" */
/**************************************/
void main(void)
{
 presentacion();

 iniciar_graficos();
 getpalette(&paleta); //guarda la informacion actual de la paleta
 pantalla();
 ponerminas(minas);
 xp=1;
 yp=1;
 xp_ant=xp;
 yp_ant=yp;
 do
 {
  opc1=getch();
  switch(opc1)
  {
   case JMINUS:
			  menu();
			  break;
   case ENTER:
			  pantalla();
			  ponerminas(minas);
			  break;
   case SUPR:
			  sinmina();
			  if(decremento==minas)
			  {
			   ganador();
			  }
			  if(tablero2[xp][yp]==1)
			  {
			   perdedor();
			  }
			  break;
   case ARRIBA:
			  yp_ant=yp;
			  xp_ant=xp;
			  yp=yp-1;  // Disminuye una posicion en la componente
			  if(yp<1)  // No deja que la posicion y de la matriz
			  {         // sea menor que cero
			   yp=yp+1;
			  }
			  mostrarcuadro();
			  break;
   case ABAJO:
			  yp_ant=yp;
			  xp_ant=xp;
			  yp=yp+1;  // Aumenta una posicion en la componente
			  if(yp>14)  // No deja que la posicion y de la matriz
			  {          // sea mayor que 13
			   yp=yp-1;
			  }
			  mostrarcuadro();
			  break;
   case DERECHA:
			  xp_ant=xp;
			  yp_ant=yp;
			  xp=xp+1;  // Aumenta una posicion en la componente
			  if(xp>20)  // No deja que la posicion x de la matriz
			  {          // sea mayor que 19
			   xp=xp-1;
			  }
			  mostrarcuadro();
			  break;
   case IZQUIERDA:
			  xp_ant=xp;
			  yp_ant=yp;
			  xp=xp-1;  // Disminuye una posicion en la componente
			  if(xp<1)  // No deja que la posicion x de la matriz
			  {         // se menor que cero
			   xp=xp+1;
			  }
			  mostrarcuadro();
			  break;
   case F3:
			  minas=menuniveles();
			  if(opc2!=CMINUS)
			  {
			   pantalla();
			   ponerminas(minas);
			  }
			  break;
   case UNO:
			  minas=minas1;
			  pantalla();
			  ponerminas(minas);
			  break;
   case DOS:
			  minas=minas2;
			  pantalla();
			  ponerminas(minas);
			  break;
   case TRES:
			  minas=minas3;
			  pantalla();
			  ponerminas(minas);
			  break;
   case CUATRO:
			  minas=personalizado();
			  pantalla();
			  ponerminas(minas);
			  break;
   case CINCO:
			  mostrar_puntuacion();
			  break;
   case SEIS:
			  reiniciar_puntuaciones();
			  break;
   case F1:
			  ayuda();
			  pantalla();
			  ponerminas(minas);
			  break;
   case F5:
			  acercade();
			  break;
   case ESC:
			  salir();
			  break;
   default:
			  break;
  }
 }while(opc1!=ESC);
 closegraph();
}

/***************************************************************************/
/*                         FUNCION INICIAR_GRAFICOS                        */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Inicia el modo grafico y comprueba si hay errores para iniciarlo.       */
/* Si el error es diferente de grOk (0) se aborta el programa con un       */
/* mensaje de error                                                        */
/***************************************************************************/

void iniciar_graficos(void)
{
 int tarjeta,modo,error;
 detectgraph(&tarjeta,&modo);
 initgraph(&tarjeta,&modo,"BGI");
 error=graphresult();
 if(error!=grOk)
 {
  clrscr();
  printf("Error de Grficos: %s\n", grapherrormsg(error));
  printf("Programa abortado... \n");
  getch();
  exit(1);
 }
}

/***************************************************************************/
/*                               FUNCION FUNC                              */
/*                                                                         */
/* No se le pasa ningun parametro, retorna un entero.                      */
/* Establece distintos modos graficos en la pantalla.                      */
/***************************************************************************/
int huge func(void)
{
 /* CAMBIA AQUI LOS VALORES PARA OBTENER DISTINTOS MODOS GRAFICOS */
 /* 2 = 640x480 */
 /* 3 = 800x600 */
 /* 4 = 1024x768 */
 return 2;
}

/***************************************************************************/
/*                            FUNCION INICIAR_SVGA                         */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Inicia el modo grafico con el driver SVGA256 y comprueba si hay errores */
/* para iniciarlo.                                                         */
/* Si el error es diferente de grOk (0) se aborta el programa con un       */
/* mensaje de error                                                        */
/***************************************************************************/
void iniciar_svga(void)
{
 int tarjeta,modo,error;
 tarjeta=DETECT;
 installuserdriver("SVGA256",func);
 initgraph(&tarjeta,&modo,"BGI");
 error=graphresult();
 if(error!=grOk)
 {
  clrscr();
  printf("Error de Grficos: %s\n", grapherrormsg(error));
  printf("Programa abortado... \n");
  getch();
  exit(1);
 }
}

/***************************************************************************/
/*                            FUNCION PRESENTACION                         */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Muestra la presentacion del programa, para mostrar los graficos se usa  */
/* el driver SVGA256 y la biblioteca "pcxbgi1.h"                           */
/***************************************************************************/
void presentacion(void)
{
 int i,j;
 PCX trex,pres3;
 iniciar_svga();
 cleardevice();
 trex.leer_cabecera(0,"trex.pcx");//358x148
 pres3.leer_cabecera(0,"pres3.pcx");//640x480

 trex.Dibujar(140,100);

 settextjustify(CENTER_TEXT,CENTER_TEXT);
 settextstyle(1,0,4);
 setcolor(DARKGRAY);
 for(i=0;i<2;i++)
 {
  outtextxy(mitx+2+i,300+2,"R E X   S O F T W A R E");
 }
 setcolor(WHITE);
 for(i=0;i<2;i++)
 {
  outtextxy(mitx+i,300,"R E X   S O F T W A R E");
 }
 delay(1000);
 //getch();
 settextstyle(1,0,2);
 setcolor(WHITE);
 for(i=0;i<2;i++)
 {
  outtextxy(mitx+i,400,"P R E S E N T A");
 }
 delay(1000);
 //getch();
 cleardevice();
 settextstyle(1,0,4);
 setcolor(DARKGRAY);
 for(i=0;i<2;i++)
 {
  outtextxy(mitx+2+i,mity+2,"\" B U S C A M I N A S \"");
 }
 setcolor(WHITE);
 for(i=0;i<2;i++)
 {
  outtextxy(mitx+i,mity,"\" B U S C A M I N A S \"");
 }

 pres3.Dibujar(0,0);

 delay(2000);
 //getch();
 closegraph();
}

/***************************************************************************/
/*                              FUNCION SALIR                              */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Imprime un mensaje de confirmacion de salida y pide una opcion; en caso */
/* de que esta opcion sea s (SMINUS) el programa termina su ejecucion, en  */
/* caso de que sea n (NMINUS) el programa continua con su ejecucion normal.*/
/***************************************************************************/
void salir(void)
{
 unsigned int sizesalida;
 void *debajosalida;
 int xsal=mitx;//mitad X del cuadro de salida en pixeles
 int ysal=mity;//mitad Y del cuadro de salida en pixeles
 /* guarda en memoria la imagen que se encuentra */
 /* debajo del cuadro de salida */
 sizesalida=imagesize(xsal-145,ysal-20,xsal+145,ysal+20);
 if((debajosalida=malloc(sizesalida))==NULL)
 {
  mensaje(mitx,mity,"Error: Memoria insuficiente para mostrar este cuadro de dilogo");
 }
 else
 {
  getimage(xsal-145,ysal-20,xsal+145,ysal+20,debajosalida);
  /* cuadro de mensaje */
  setfillstyle(1,LIGHTGRAY);
  bar(xsal-140,ysal-15,xsal+140,ysal+15);
  /* Bordes blancos del cuadro de mensaje */
  setlinestyle(0,0,1);
  setcolor(WHITE);
  line(xsal-141,ysal-16,xsal+141,ysal-16);
  line(xsal-141,ysal-16,xsal-141,ysal+16);
  rectangle(xsal-141+6,ysal+13,xsal+141-4,ysal-11);
  /* Bordes grises del cuadro de mensaje */
  setcolor(DARKGRAY);
  line(xsal-141,ysal+16,xsal+141,ysal+16);
  line(xsal+141,ysal-16,xsal+141,ysal+16);
  rectangle(xsal-141+5,ysal+12,xsal+141-5,ysal-12);
  /* texto que se mostrara en el cuadro de mensaje */
  settextjustify(LEFT_TEXT,CENTER_TEXT);
  settextstyle(0,0,1);
  setcolor(WHITE);
  outtextxy(xsal-130,ysal+1,"Confirma que Desea Salir(S/N):");
  setcolor(BLACK);
  outtextxy(xsal-131,ysal,"Confirma que Desea Salir(S/N):");
  /* barra negra donde se escribiran las opciones */
  setfillstyle(1,BLACK);bar(xsal+109,ysal-8,xsal+128,ysal+8);
  /* Bordes de la barra donde se escriben las opciones */
  setlinestyle(0,0,1);
  setcolor(DARKGRAY);
  line(xsal+109,ysal-9,xsal+129,ysal-9);
  line(xsal+109,ysal-9,xsal+109,ysal+9);
  setcolor(WHITE);
  line(xsal+109,ysal+9,xsal+129,ysal+9);
  line(xsal+129,ysal-9,xsal+129,ysal+9);
  /* Recibe una opcion del teclado */
  setcolor(WHITE);
  opc_salida=getch();
  switch(opc_salida)
  {
   case SMINUS:
				outtextxy(mitx+116,mity,"S");
				opc1=ESC;
				break;
   case NMINUS:
				outtextxy(mitx+116,mity,"N");
				opc1=ENTER;
				break;
   default:
				outtextxy(mitx+116,mity,"S");
				opc1=ESC;
				break;
  }
  getch();
  /* Muestra la imagen guardada y libera la memoria */
  /* que fue reservada para el cuadro de salida */
  putimage(xsal-145,ysal-20,debajosalida,COPY_PUT);
  free(debajosalida);
 }
}

/***************************************************************************/
/*                           FUNCION PANTALLA                              */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Dibuja la pantalla principal del juego, y averigua e imprime el nivel   */
/* del juego (Principiante, Avanzado o Experto).                           */
/***************************************************************************/
void pantalla(void)
{
 int i,j; //variables para dibujar el tablero de juego
 opc3=CMINUS;
 puntos=0;
 cleardevice();
 //setpalette(1,8);
 setlinestyle(0,0,1);
 /* Fondo negro */
 setfillstyle(1,BLACK);
 bar(0,0,maxx,maxy);
 /* Fondo gris (cuadros que representan cada casilla) */
 setfillstyle(1,LIGHTGRAY);
 bar(20,38,619,457);
 /* barra de titulo */
 setfillstyle(1,BLUE);
 bar(20,0,maxx-19,20);
 /* barra del menu */
 setfillstyle(1,LIGHTGRAY);
 bar(20,21,maxx-19,37);
 setcolor(DARKGRAY);
 rectangle(20,21,maxx-19,37);
 /* barra de informacion */
 setfillstyle(1,DARKGRAY);
 bar(20,maxy-20,maxx-19,maxy-2);
 setcolor(LIGHTGRAY);
 rectangle(20,maxy-20,maxx-19,maxy-2);
 /* texto de los lados */
 setcolor(LIGHTGRAY);
 settextstyle(0,1,1); //justificacion vertical
 settextjustify(CENTER_TEXT,CENTER_TEXT);
 outtextxy(8,mity,          "Buscaminas  Buscaminas  Buscaminas  Buscaminas  Buscaminas");
 outtextxy(maxx-8,mity,"Buscaminas  Buscaminas  Buscaminas  Buscaminas  Buscaminas");
 /* Titulo */
 settextjustify(CENTER_TEXT,CENTER_TEXT);
 settextstyle(0,0,1);
 setcolor(WHITE);
 rectangle(maxx-36,5,maxx-24,17);
 setcolor(LIGHTGRAY);
 outtextxy(maxx-30,12,"X");
 settextstyle(4,0,1);
 setcolor(BLACK);
 outtextxy(mitx+4,9,"B u s c a m i n a s");
 setcolor(YELLOW);
 outtextxy(mitx,7,"B u s c a m i n a s");
 /* Texto de la barra del  menu */
 settextstyle(0,0,1);
 settextjustify(LEFT_TEXT,CENTER_TEXT);
 setcolor(RED);
 outtextxy(30,29,"J                             F1                      ESC                ");
 setcolor(BLACK);
 outtextxy(30,29,"  u e g o          A y u d a <  >          S a l i r <   >               ");
 /* Texto de la informacion */
 outtextxy(30,maxy-10,"Minas: ");
 outtextxy(mitx-140,maxy-10,"Nivel: ");
 outtextxy(mitx+70,maxy-10,"Puntuacion: ");
 setcolor(WHITE);
 outtextxy(mitx+180,maxy-10,"0");
 if(minas==10)
  outtextxy(mitx-80,maxy-10,"Principiante");
 if(minas==30)
  outtextxy(mitx-80,maxy-10,"Avanzado");
 if(minas==50)
  outtextxy(mitx-80,maxy-10,"Experto");
 if(minas!=10 && minas!=30 && minas!=50)
  outtextxy(mitx-80,maxy-10,"Personalizado");
 /* Cuadros del tablero de juego */
 setlinestyle(0,0,1);
 for(i=38;i<=maxy-23;i=i+30)
 {
  for(j=20;j<=maxx-19;j=j+30)
  {
   setcolor(WHITE);
   line(j+1,i+1,j+1,maxy-21);
   line(j+1,i+1,maxx-21,i+1);
   setcolor(BLACK);
   line(j,i,j,maxy-21);
   line(j,i,maxx-19,i);
  }
 }
 line(maxx-19,38,maxx-19,maxy-21);
 line(20,maxy-21,maxx-19,maxy-21);
 /* rectangulo inicial */
 xg=(1*30)+5;
 yg=(1*30)+25;
 setlinestyle(0,0,1);
 setcolor(LIGHTRED);
 rectangle(xg-15,yg-17,xg+15,yg+13);
 /* borde que rodea la pantalla de juego */
 setcolor(WHITE);
 setlinestyle(0,0,3);
 rectangle(18,0,maxx-17,maxy);
}

/***************************************************************************/
/*                              FUNCION MENU                               */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Dibuja el menu principal del juego, al presionar la tecla indicada      */
/* dirige el codigo a las respectivas funciones que se muestran en pantalla*/
/* (Nuevo, Niveles, Ayuda, Acerca de, Cancelar o Salir).                   */
/***************************************************************************/
void menu(void)
{
 unsigned int sizemenu;
 void *debajomenu;
 /* esquina superior izquierda del menu */
 int x1=20;
 int y1=38;
 /* esquina inferior derecha del menu */
 int x2=160;
 int y2=230;
 opc3=CMINUS;
 sizemenu=imagesize(x1-5,y1-5,x2+5,y2+5);
 if((debajomenu=malloc(sizemenu))==NULL)
 {
  mensaje(mitx,mity,"Error: Memoria insuficiente para mostrar este cuadro de dilogo");
 }
 else
 {
  getimage(x1-5,y1-5,x2+5,y2+5,debajomenu);
  /* fondo */
  setfillstyle(1,LIGHTGRAY);
  bar(x1,y1,x2,y2);
  /* bordes blancos */
  setlinestyle(0,0,1);
  setcolor(WHITE);
  line(x1,y1,x1,y2);
  line(x1,y1,x2,y1);
  rectangle(x1+6,y1+6,x2-4,y2-4);
  /* bordes grises */
  setcolor(DARKGRAY);
  line(x2,y1,x2,y2);
  line(x1,y2,x2,y2);
  rectangle(x1+5,y1+5,x2-5,y2-5);
  /* texto de las opciones que se muestran en el menu */
  settextjustify(LEFT_TEXT,CENTER_TEXT);
  settextstyle(0,0,1);
  setcolor(RED);
  outtextxy(x1+10,y1+15, " N");
  outtextxy(x1+10,y1+45, "   v");
  outtextxy(x1+10,y1+75, "  y");
  outtextxy(x1+10,y1+105," A");
  outtextxy(x1+10,y1+135," C");
  outtextxy(x1+10,y1+165," S");
  setcolor(BLACK);
  outtextxy(x1+18,y1+15, " uevo    ENTER");
  outtextxy(x1+18,y1+45, "Ni eles    F3 ");
  outtextxy(x1+124,y1+45,"");
  outtextxy(x1+18,y1+75, "A uda       F1");
  outtextxy(x1+18,y1+105," cerca de   F5");
  outtextxy(x1+18,y1+135," ancelar");
  outtextxy(x1+18,y1+165," alir      ESC");
  /* lee una opcion del teclado*/
  do
  {
   opc2=getch();
   /* muestra la imagen que esta dejo del menu y libera memoria */
   putimage(x1-5,y1-5,debajomenu,COPY_PUT);
   free(debajomenu);
   /**/
   switch(opc2)
   {
	case ENTER:
			pantalla();
			ponerminas(minas);
			opc2=CMINUS;
			break;
	case NMINUS:
			pantalla();
			ponerminas(minas);
			opc2=CMINUS;
			break;
	case VMINUS:
			minas=menuniveles();
			if(opc2!=CMINUS)
			{
			 pantalla();
			 ponerminas(minas);
			}
			opc2=CMINUS;
			break;
	case F3:
			minas=menuniveles();
			if(opc2!=CMINUS)
			{
			 pantalla();
			 ponerminas(minas);
			}
			opc2=CMINUS;
			break;
	case YMINUS:
			ayuda();
			pantalla();
			ponerminas(minas);
			opc2=CMINUS;
			break;
	case F1:
			ayuda();
			pantalla();
			ponerminas(minas);
			opc2=CMINUS;
			break;
	case AMINUS:
			acercade();
			opc2=CMINUS;
			break;
	case F5:
			acercade();
			opc2=CMINUS;
			break;
	case CMINUS:
			if(opc3==ENTER || opc3==JMINUS)
			{
			 pantalla();
			 ponerminas(minas);
			}
			break;
	case ESC:
			opc1=ESC;
			break;
	case SMINUS:
			opc1=ESC;
			break;
	default:
			opc2=CMINUS;
			break;
   }
  }while(opc2!=SMINUS && opc2!=CMINUS && opc2!=ESC);
 }
}

/***************************************************************************/
/*                           FUNCION MENUNIVELES                           */
/*                                                                         */
/* No se le pasa ningun parametro, retorna el numero de minas que va a     */
/* tener el tablero.                                                       */
/* Dibuja el menu de los niveles que tiene el juego, asigna el numero de   */
/* minas e imprime el respectivo nivel de juego (Principiante, Avanzado o  */
/* Experto), o dirige el codigo a otra funcion (Personalizado).            */
/***************************************************************************/
int menuniveles(void)
{
 unsigned int sizemenuniveles;
 void *debajomenuniveles;
 /* esquina superior izquierda del menuniveles */
 int x1=161;
 int y1=68;
 /* esquina inferior derecha del menuniveles */
 int x2=310;
 int y2=260;
 sizemenuniveles=imagesize(x1-5,y1-5,x2+5,y2+5);
 if((debajomenuniveles=malloc(sizemenuniveles))==NULL)
 {
  mensaje(mitx,mity,"Error: Memoria insuficiente para mostrar este cuadro de dilogo");
 }
 else
 {
  getimage(x1-5,y1-5,x2+5,y2+5,debajomenuniveles);
  /* fondo */
  setfillstyle(1,LIGHTGRAY);
  bar(x1,y1,x2,y2);
  /* bordes blancos */
  setlinestyle(0,0,1);
  setcolor(WHITE);
  line(x1,y1,x1,y2);
  line(x1,y1,x2,y1);
  rectangle(x1+6,y1+6,x2-4,y2-4);
  /* bordes grises */
  setcolor(DARKGRAY);
  line(x2,y1,x2,y2);
  line(x1,y2,x2,y2);
  rectangle(x1+5,y1+5,x2-5,y2-5);
  /* texto de las opciones que se muestran en el menu */
  settextjustify(LEFT_TEXT,CENTER_TEXT);
  settextstyle(0,0,1);
  setcolor(RED);
  outtextxy(x1+10,y1+15, " P");
  outtextxy(x1+10,y1+45, " A");
  outtextxy(x1+10,y1+75, " E");
  outtextxy(x1+10,y1+105,"   r");
  outtextxy(x1+10,y1+135,"  u");
  outtextxy(x1+10,y1+165,"   i");
  setcolor(BLACK);
  outtextxy(x1+18,y1+15, " rincipiante  1");
  outtextxy(x1+18,y1+45, " vanzado      2");
  outtextxy(x1+18,y1+75, " xperto       3");
  outtextxy(x1+18,y1+105,"Pe sonalizado 4");
  outtextxy(x1+18,y1+135,"P ntuaciones  5");
  outtextxy(x1+18,y1+165,"Re niciar Pun 6");
  /* lee una una opcion del teclado */
  opc_niveles=getch();
  /* muestra la imagen que esta dejo del menuniveles y libera memoria */
  putimage(x1-5,y1-5,debajomenuniveles,COPY_PUT);
  free(debajomenuniveles);
  setcolor(WHITE);
  switch(opc_niveles)
  {
   case PMINUS:
			minas=minas1;
			break;
   case UNO:
			minas=minas1;
			break;
   case AMINUS:
			minas=minas2;
			break;
   case DOS:
			minas=minas2;
			break;
   case EMINUS:
			minas=minas3;
			break;
   case TRES:
			minas=minas3;
			break;
   case RMINUS:
			minas=personalizado();
			break;
   case CUATRO:
			minas=personalizado();
			break;
   case UMINUS:
			mostrar_puntuacion();
			minas=minas;
			opc2=CMINUS;
			break;
   case CINCO:
			mostrar_puntuacion();
			minas=minas;
			opc2=CMINUS;
			break;
   case IMINUS:
			reiniciar_puntuaciones();
			minas=minas;
			opc2=CMINUS;
			break;
   case SEIS:
			reiniciar_puntuaciones();
			minas=minas;
			opc2=CMINUS;
			break;
   default:
			minas=minas;
			opc2=CMINUS;
			break;
  }
 }
 return(minas);
}

/***************************************************************************/
/*                       FUNCION REINICIAR_PUNTUACIONES                    */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Reinicia todas las puntuaciones.                                        */
/***************************************************************************/
void reiniciar_puntuaciones(void)
{
 /* se abre el archivo para escritura */
 if((fp=fopen("PUNTAJE.TXT","w"))==NULL)
 {
  mensaje(mitx,mity,"Error: No se encuentra el archivo \"Puntaje.txt\"");
 }
 else
 {
  /* se escriben los nombres y las puntuaciones en el archivo */
  fprintf(fp,"Anonimo\n");
  fprintf(fp,"100000000\n");
  fprintf(fp,"Anonimo\n");
  fprintf(fp,"100000000\n");
  fprintf(fp,"Anonimo\n");
  fprintf(fp,"100000000\n");
  fprintf(fp,"Anonimo\n");
  fprintf(fp,"100000000\n");
  fprintf(fp,"Anonimo\n");
  fprintf(fp,"100000000\n");
  /* se cierra el archivo y se muestra un mensaje de confirmacion */
  fclose(fp);
  mensaje(mitx,mity,"Puntuaciones Reiniciadas");
 }
}

/***************************************************************************/
/*                        FUNCION MOSTRAR_PUNTUACION                             */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Lee las puntuaciones desde el archivo y las muestra en pantalla         */
/***************************************************************************/
void mostrar_puntuacion(void)
{
 unsigned int sizepuntuacion;
 void *debajopuntuacion;
 /* esquina superior izquierda de las puntuaciones */
 int x1=mitx-135;
 int y1=mity-90;
 /* esquina inferior derecha de las puntuaciones */
 int x2=mitx+135;
 int y2=mity+80;
 /* puntajes que se muestran en pantalla con sus respectivos nombres */
 puntajes mostrar[5];
 /* se abre el archivo para lectura */
 if((fp=fopen("PUNTAJE.TXT","r"))==NULL)
 {
  mensaje(mitx,mity,"Error: No se encuentra el archivo \"Puntaje.txt\"");
  mensaje(mitx,mity,"Reinicie las puntuaciones para crear el archivo");
 }
 else
 {
  /* se leen los nombres y las puntuaciones desde el archivo */
  fscanf(fp,"%s\n",&mostrar[0].nombre);
  fscanf(fp,"%s\n",&mostrar[0].puntaje);
  fscanf(fp,"%s\n",&mostrar[1].nombre);
  fscanf(fp,"%s\n",&mostrar[1].puntaje);
  fscanf(fp,"%s\n",&mostrar[2].nombre);
  fscanf(fp,"%s\n",&mostrar[2].puntaje);
  fscanf(fp,"%s\n",&mostrar[3].nombre);
  fscanf(fp,"%s\n",&mostrar[3].puntaje);
  fscanf(fp,"%s\n",&mostrar[4].nombre);
  fscanf(fp,"%s\n",&mostrar[4].puntaje);
  /* se cierra el archivo */
  fclose(fp);

 sizepuntuacion=imagesize(x1-5,y1-5,x2+5,y2+5);
 if((debajopuntuacion=malloc(sizepuntuacion))==NULL)
 {
  mensaje(mitx,mity,"Error: Memoria insuficiente para mostrar este cuadro de dilogo");
 }
 else
 {
  getimage(x1-5,y1-5,x2+5,y2+5,debajopuntuacion);
  /* fondo del boton */
  setfillstyle(1,LIGHTGRAY);
  bar(x1,y1,x2,y2);
  /* bordes */
  setlinestyle(0,0,1);
  setcolor(WHITE);
  line(x1,y1,x1,y2);
  line(x1,y1,x2,y1);
  rectangle(x1+6,y1+6,x2-4,y2-4);//borde grande
  rectangle(x1+6,y1+23,x2-4,y2-4);//borde puntajes
  rectangle(x1+6,y1+23,x1+171,y2-4);//borde nombres
  rectangle(x1+6,y1+23,x1+35,y2-4);//borde numeros
  rectangle(x1+6,y1+23,x2-4,y1+37);//borde titulos
  setcolor(DARKGRAY);
  line(x2,y1,x2,y2);
  line(x1,y2,x2,y2);
  rectangle(x1+5,y1+5,x2-5,y2-5); //borde grande
  rectangle(x1+5,y1+22,x2-5,y2-5);//borde puntajes
  rectangle(x1+5,y1+22,x1+170,y2-5);//borde nombres
  rectangle(x1+5,y1+22,x1+34,y2-5);//borde numeros
  rectangle(x1+5,y1+22,x2-5,y1+36);//borde titulos
  /* texto del boton */
  settextjustify(LEFT_TEXT,CENTER_TEXT);
  settextstyle(0,0,1);
  setcolor(BLUE);
  outtextxy(x1+11,y1+31, "No.");
  outtextxy(x1+79,y1+31, "Nombre");
  outtextxy(x1+195,y1+31,"Puntaje");
  setcolor(WHITE);
  outtextxy(x1+47,y1+16, "P U N T U A C I O N E S");
  outtextxy(x1+10,y1+30, "No.");
  outtextxy(x1+78,y1+30, "Nombre");
  outtextxy(x1+194,y1+30,"Puntaje");
  setcolor(BLACK);
  outtextxy(x1+46,y1+15, "P U N T U A C I O N E S");
  outtextxy(x1+14,y1+50, "1.");
  outtextxy(x1+14,y1+75, "2.");
  outtextxy(x1+14,y1+100,"3.");
  outtextxy(x1+14,y1+125,"4.");
  outtextxy(x1+14,y1+150,"5.");
  /* se escriben los nombres y las puntuaciones en el boton */
  setcolor(BLACK);
  outtextxy(x1+42,y1+50,mostrar[0].nombre);
  outtextxy(x1+178,y1+50,mostrar[0].puntaje);
  outtextxy(x1+42,y1+75,mostrar[1].nombre);
  outtextxy(x1+178,y1+75,mostrar[1].puntaje);
  outtextxy(x1+42,y1+100,mostrar[2].nombre);
  outtextxy(x1+178,y1+100,mostrar[2].puntaje);
  outtextxy(x1+42,y1+125,mostrar[3].nombre);
  outtextxy(x1+178,y1+125,mostrar[3].puntaje);
  outtextxy(x1+42,y1+150,mostrar[4].nombre);
  outtextxy(x1+178,y1+150,mostrar[4].puntaje);
  getch();
  putimage(x1-5,y1-5,debajopuntuacion,COPY_PUT);
  free(debajopuntuacion);
 }
 }
}

/***************************************************************************/
/*                          FUNCION PERSONALIZADO                          */
/*                                                                         */
/* No se le pasa ningun parametro, retorna el numero de minas que va a     */
/* tener el tablero en el nivel personalizado.                             */
/* Muestra un mensaje pidiendole al usuario el numero de minas que quiere  */
/* que tenga este nivel, e imprime el nivel de juego (Personalizado).      */
/***************************************************************************/
int personalizado(void)
{
 /* opcp: lee una opcion del teclado(numero de minas que tendra este nivel)*/
 /* A: Arreglo para guardar cada numero(3 numeros como maximo) */
 /* *c: Apunta al primer elemento de A para convertir la cadena a un entero*/
 char opcp,A[3],*c;
 /* pos: Posicion en el arreglo A (0,1  2) */
 int pos=0;
 /* xper: Mitad X del boton personalizado */
 int xper=412;
 /* yper: Mitad Y del boton personalizado */
 int yper=184;
 /* fondo del boton */
 setfillstyle(1,LIGHTGRAY);
 bar(xper-100,yper-15,xper+100,yper+15);
 /* bordes blancos */
 setlinestyle(0,0,1);
 setcolor(WHITE);
 line(xper-101,yper-16,xper+101,yper-16);
 line(xper-101,yper-16,xper-101,yper+16);
 rectangle(xper-101+6,yper+13,xper+101-4,yper-11);
 /* bordes grises */
 setcolor(DARKGRAY);
 line(xper-101,yper+16,xper+101,yper+16);
 line(xper+101,yper-16,xper+101,yper+16);
 rectangle(xper-101+5,yper+12,xper+101-5,yper-12);
 /* mensaje que se muestra en el boton */
 settextstyle(0,0,1);
 settextjustify(LEFT_TEXT,CENTER_TEXT);
 setcolor(WHITE);outtextxy(xper-90,yper+1,"Numero de Minas: ");
 setcolor(BLACK);outtextxy(xper-91,yper,"Numero de Minas: ");
 /* barra negra donde el usuario digitara el numero de minas */
 xper=xper+68;
 setfillstyle(1,BLACK);bar(xper-20,yper-8,xper+20,yper+8);
 /* bordes de la barra negra */
 setlinestyle(0,0,1);
 setcolor(DARKGRAY);
 line(xper-21,yper-9,xper+21,yper-9);
 line(xper-21,yper-9,xper-21,yper+9);
 setcolor(WHITE);
 line(xper-21,yper+9,xper+21,yper+9);
 line(xper+21,yper-9,xper+21,yper+9);

 xper=465;
 yper=184;
 /* lee una opcion del teclado y la escribe en la barra negra */
 settextstyle(0,0,1);
 setcolor(WHITE);
 do
 {
  opcp=getch();
  switch(opcp)
  {
   case CERO:
			 if(pos<=2)
			 {
			  outtextxy(xper,yper,"0");
			  xper=xper+10;
			  A[pos]='0';
			  pos=pos+1;
			 }
			 break;
   case UNO:
			 if(pos<=2)
			 {
			  outtextxy(xper,yper,"1");
			  xper=xper+10;
			  A[pos]='1';
			  pos=pos+1;
			 }
			 break;
   case DOS:
			 if(pos<=2)
			 {
			  outtextxy(xper,yper,"2");
			  xper=xper+10;
			  A[pos]='2';
			  pos=pos+1;
			 }
			 break;
   case TRES:
			 if(pos<=2)
			 {
			  outtextxy(xper,yper,"3");
			  xper=xper+10;
			  A[pos]='3';
			  pos=pos+1;
			 }
			 break;
   case CUATRO:
			 if(pos<=2)
			 {
			  outtextxy(xper,yper,"4");
			  xper=xper+10;
			  A[pos]='4';
			  pos=pos+1;
			 }
			 break;
   case CINCO:
			 if(pos<=2)
			 {
			  outtextxy(xper,yper,"5");
			  xper=xper+10;
			  A[pos]='5';
			  pos=pos+1;
			 }
			 break;
   case SEIS:
			 if(pos<=2)
			 {
			  outtextxy(xper,yper,"6");
			  xper=xper+10;
			  A[pos]='6';
			  pos=pos+1;
			 }
			 break;
   case SIETE:
			 if(pos<=2)
			 {
			  outtextxy(xper,yper,"7");
			  xper=xper+10;
			  A[pos]='7';
			  pos=pos+1;
			 }
			 break;
   case OCHO:
			 if(pos<=2)
			 {
			  outtextxy(xper,yper,"8");
			  xper=xper+10;
			  A[pos]='8';
			  pos=pos+1;
			 }
			 break;
   case NUEVE:
			 if(pos<=2)
			 {
			  outtextxy(xper,yper,"9");
			  xper=xper+10;
			  A[pos]='9';
			  pos=pos+1;
			 }
			 break;
   case BACKSPACE:
			 if(pos>0)
			 {
			  setfillstyle(1,BLACK);
			  xper=xper-10;
			  bar(xper-2,yper-6,xper+10,yper+6);
			  pos=pos-1;
			  A[pos]='\0';
			 }
			 break;
   case ENTER:
			 setfillstyle(1,0);
			 c=&A[0];
			 minas4=atoi(c);
			 if((minas4<0) || (minas4>279))
			 {
			  mensaje(410,184,"Ingrese un valor entre 0 y 279");
			  minas4=personalizado();
			 }
			 break;
   default:
			 break;
  }
 }while(opcp!=ENTER);
 return(minas4);
}

/***************************************************************************/
/*                              FUNCION CARITA                             */
/*                                                                         */
/* Se le pasa la posicion X y Y de la matriz donde desea dibujarse la      */
/* carita feliz y el color de fondo de la carita. Estos valores se leen en */
/* las variables xc y yc.                                                  */
/* Dibuja la carita feliz en la posicion que se le indique.                */
/***************************************************************************/
void carita(int xc,int yc,int cfondo)
{
 xg=(xc*30)+5;
 yg=(yc*30)+25;
 /* fondo */
 setfillstyle(1,cfondo);
 bar(xg-14,yg-16,xg+14,yg+12);
 /* circulo de la cara */
 setlinestyle(0,0,1);
 setcolor(BLACK);
 setfillstyle(1,YELLOW);
 fillellipse(xg,yg-2,10,10);
 /* boca */
 arc(xg,yg-1,180,360,5);
 arc(xg,yg,180,360,5);
 /* ojos */
 setlinestyle(0,0,3);
 line(xg-4,yg-4,xg-2,yg-4);
 line(xg+2,yg-4,xg+4,yg-4);
}

/***************************************************************************/
/*                            FUNCION CARITAMALA                           */
/*                                                                         */
/* Se le pasa la posicion X y Y de la matriz donde desea dibujarse la      */
/* carita triste y el color de fondo de la carita. Estos valores se leen   */
/* en las variables xc y yc.                                               */
/* Dibuja la carita triste en la posicion que se le indique.               */
/***************************************************************************/
void caritamala(int xc,int yc,int cfondo)
{
 xg=(xc*30)+5;
 yg=(yc*30)+25;
 /* fondo */
 setfillstyle(1,cfondo);
 bar(xg-14,yg-16,xg+14,yg+12);
 /* circulo de la cara */
 setlinestyle(0,0,1);
 setcolor(BLACK);
 setfillstyle(1,YELLOW);
 fillellipse(xg,yg-2,10,10);
 /* boca */
 arc(xg,yg+4,0,180,5);
 /* ojos */
 line(xg-4,yg-4,xg-2,yg-6);
 line(xg+2,yg-4,xg+4,yg-6);
 line(xg-4,yg-6,xg-2,yg-4);
 line(xg+2,yg-6,xg+4,yg-4);
}

/***************************************************************************/
/*                              FUNCION GANADOR                            */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Dibuja la pantalla que aparece cuando se gana el juego.                 */
/***************************************************************************/
void ganador(void)
{
 randomize();
 /* espera a que se presione Enter o J minscula */
 do
 {
  /* dibuja caritas aleatoriamente en la pantalla, */
  /* hasta que se presione una tecla */
  do
  {
   carita(random(20)+1,random(14)+1,random(15)+1);
  }while(!kbhit());
  opc3=getch();
 }while(opc3!=ENTER && opc3!=JMINUS);

 guardar_puntuacion();
 /* dibuja pantalla y pone minas */
 pantalla();
 ponerminas(minas);
}

/***************************************************************************/
/*                        FUNCION GUARDAR_PUNTUACION                       */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Guarda la menor puntuacion fichero.                                     */
/***************************************************************************/
void guardar_puntuacion(void)
{
 /* La ultima puntuacion obtenida se guarda con su respectivo nombre */
 /* en la ultima del arreglo guardar (guardar[5]) y se ordena el     */
 /* arreglo de menor a mayor por el metodo de la burbuja.            */
 /* Variables para recorrer el arreglo */
 int cont,pivote;
 /* Guarda los puntajes como tipo long int en un arreglo temporal */
 long int temp[6],num_temp;;
 /* Variables temporalmente para cambiar los puntajes y los nombres */
 char *cad_temp,temp_nombre[16];
 /* Arreglo para guardar los puntajes con el nombre respectivo */
 puntajes guardar[6];
 /* se abre el archivo para lectura */
 if((fp=fopen("PUNTAJE.TXT","r"))==NULL)
 {
  mensaje(mitx,mity,"Error: No se encuentra el archivo \"Puntaje.txt\"");
  mensaje(mitx,mity,"Reinicie las puntuaciones para crear el archivo");
 }
 else
 {
  /* se leen los puntajes y los nombres el archivo */
  fscanf(fp,"%s\n",&guardar[0].nombre);
  fscanf(fp,"%s\n",&guardar[0].puntaje);
  fscanf(fp,"%s\n",&guardar[1].nombre);
  fscanf(fp,"%s\n",&guardar[1].puntaje);
  fscanf(fp,"%s\n",&guardar[2].nombre);
  fscanf(fp,"%s\n",&guardar[2].puntaje);
  fscanf(fp,"%s\n",&guardar[3].nombre);
  fscanf(fp,"%s\n",&guardar[3].puntaje);
  fscanf(fp,"%s\n",&guardar[4].nombre);
  fscanf(fp,"%s\n",&guardar[4].puntaje);
  fclose(fp);
  /* se guarda la puntuacion obtenida en la ultima posicion del arreglo */
  strcpy(guardar[5].puntaje,puntuacion);
  /* se convierten las puntuaciones a long int */
  for(cont=0;cont<=5;cont++)
  {
   temp[cont]=atol(guardar[cont].puntaje);
  }
  /* si la puntuacion obtenida es menor a las anteriores, se guarda el */
  /* nombre y se copia a la ultima posicion del arreglo.               */
  if(temp[5]<=temp[4])
  {
   guardar_nombre();
  }
  strcpy(guardar[5].nombre,nombre_puntuacion);
  /* pivote se ubica en la ultima posicion del arreglo y se disminuira */
  /* de uno en uno hasta que el arreglo se haya ordenado. */
  pivote=5;
  do
  {
   /* se recorre el arreglo hasta el pivote*/
   for(cont=0;cont<pivote;cont++)
   {
	/* se compara cada posicion con la siguiente , si es mayor se */
	/* intercambian. */
	if(temp[cont]>=temp[cont+1])
	{
	 num_temp=temp[cont];
	 temp[cont]=temp[cont+1];
	 temp[cont+1]=num_temp;
	 strcpy(cad_temp,guardar[cont].puntaje);
	 strcpy(guardar[cont].puntaje,guardar[cont+1].puntaje);
	 strcpy(guardar[cont+1].puntaje,cad_temp);
	 strcpy(temp_nombre,guardar[cont].nombre);
	 strcpy(guardar[cont].nombre,guardar[cont+1].nombre);
	 strcpy(guardar[cont+1].nombre,temp_nombre);
	}
   }
   /* despues de recorrer todo el arreglo de disminuye el pivote y se */
   /* vuelve arecorrer de nuevo hasta que el pivote sea menor que cero */
   pivote=pivote-1;
  }while(pivote>=0);
 }//fin else para lectura
 /* se abre el archivo para escitura */
 if((fp = fopen("PUNTAJE.TXT","w")) == NULL)
 {
  mensaje(mitx,mity,"Error: No se encuentra el archivo \"Puntaje.txt\"");
  mensaje(mitx,mity,"Reinicie las puntuaciones para crear el archivo");
 }
 else
 {
  /* se escriben los puntajes y los nombres en el archivo */
  fprintf(fp,"%s\n",guardar[0].nombre);
  fprintf(fp,"%s\n",guardar[0].puntaje);
  fprintf(fp,"%s\n",guardar[1].nombre);
  fprintf(fp,"%s\n",guardar[1].puntaje);
  fprintf(fp,"%s\n",guardar[2].nombre);
  fprintf(fp,"%s\n",guardar[2].puntaje);
  fprintf(fp,"%s\n",guardar[3].nombre);
  fprintf(fp,"%s\n",guardar[3].puntaje);
  fprintf(fp,"%s\n",guardar[4].nombre);
  fprintf(fp,"%s\n",guardar[4].puntaje);
  fclose(fp);
 }//fin else para escritura
}

/***************************************************************************/
/*                          FUNCION GUARDAR_NOMBRE                         */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Guarda el nombre de la persona que obtuvo una puntuacion en una cadena  */
/* de caracteres.                                                          */
/***************************************************************************/
void guardar_nombre(void)
{
 /* pos: Posiciones del arreglo nombre_puntuacion para escribir las letras */
 /* posg: Posicion grafica donde se empezaran a escribir las letras */
 /* xnom: Mitad X del boton */
 /* ynom: Mitad Y del boton */
 int pos,posg,xnom,ynom;
 /* *cad_aux: Cadena de caracteres para escribir las letras en pantalla */
 char *cad_aux;
 /* opc_nom: Variable que guarda un caracter leido desde el teclado para */
 /*          escribirlo en pantalla */
 char opcnom;
 unsigned int sizenom;
 void *debajonom;
 xnom=mitx;
 ynom=mity+10;
 cad_aux=(char *)malloc(16*sizeof(char));
 sizenom=imagesize(xnom-100,ynom-20,xnom+100,ynom+20);
 if((debajonom=malloc(sizenom))==NULL)
 {
  mensaje(mitx,mity,"Error: Memoria insuficiente para mostrar este cuadro de dilogo");
 }
 else
 {
  getimage(xnom-100,ynom-20,xnom+100,ynom+20,debajonom);
  /* cuadro de mensaje de la ayuda */
  setfillstyle(1,LIGHTGRAY);bar(xnom-115,ynom-15,xnom+115,ynom+15);
  /* Bordes blancos del cuadro de mensaje */
  setlinestyle(0,0,1);
  setcolor(WHITE);
  line(xnom-115,ynom-16,xnom+115,ynom-16);
  line(xnom-115,ynom-16,xnom-115,ynom+16);
  rectangle(xnom-115+6,ynom+13,xnom+115-4,ynom-11);
  /* Bordes grises del cuadro de mensaje */
  setcolor(DARKGRAY);
  line(xnom-115,ynom+16,xnom+115,ynom+16);
  line(xnom+115,ynom-16,xnom+115,ynom+16);
  rectangle(xnom-115+5,ynom+12,xnom+115-5,ynom-12);
  /* texto que se mostrara en el cuadro de mensaje */
  settextjustify(LEFT_TEXT,CENTER_TEXT);
  settextstyle(0,0,1);
  setcolor(WHITE);outtextxy(xnom-102,ynom+1,"Nombre:");
  setcolor(BLACK);outtextxy(xnom-103,ynom,"Nombre:");
  /* barra negra donde se escribiran las opcionses */
  setfillstyle(1,BLACK);bar(xnom-45,ynom-8,xnom+103,ynom+8);
  /* Bordes de la barra donde se escriben las opciones */
  setlinestyle(0,0,1);
  setcolor(DARKGRAY);
  line(xnom-45,ynom-9,xnom+103,ynom-9);
  line(xnom-45,ynom-9,xnom-45,ynom+9);
  setcolor(WHITE);
  line(xnom-45,ynom+9,xnom+103,ynom+9);
  line(xnom+103,ynom-9,xnom+103,ynom+9);
  /**/
  setcolor(DARKGRAY);
  outtextxy(xnom-31,ynom+3,"_______________");
  pos=0;
  posg=xnom-31;
  setcolor(WHITE);
  strcpy(nombre_puntuacion,"               ");
  do
  {
   opcnom=getch();
   switch(opcnom)
   {
	case BACKSPACE:
			   if(pos>0)
			   {
				pos=pos-1;
				nombre_puntuacion[pos]='\0';
				setfillstyle(1,BLACK);
				bar(posg+(pos*8),ynom-6,posg+((pos*8)+8),ynom+4);
			   }
			   if(pos==0)
			   {
				nombre_puntuacion[pos]='\0';
				setfillstyle(1,BLACK);
				bar(posg+(pos*8),ynom-6,posg+((pos*8)+8),ynom+4);
			   }
			   break;
	case SPACE:
			   if(pos<=14)
			   {
				nombre_puntuacion[pos]=PUNTOARRIBA;
				strcpy(cad_aux,nombre_puntuacion);
				outtextxy(posg+(pos*8),ynom,cad_aux+pos);
				pos++;
			   }
			   break;
	case ENTER:
			   if(pos==0)
			   {
				strcpy(nombre_puntuacion,"Anonimo");
			   }
			   else
			   {
				nombre_puntuacion[pos]='\0';
			   }
			   break;
	default:
			   if(pos<=14)
			   {
				nombre_puntuacion[pos]=opcnom;
				strcpy(cad_aux,nombre_puntuacion);
				outtextxy(posg+(pos*8),ynom,cad_aux+pos);
				pos++;
			   }
			   break;
   }
  }while(opcnom!=ENTER);
  free(cad_aux);
  putimage(xnom-100,ynom-20,debajonom,COPY_PUT);
  free(debajonom);
 }
}

/***************************************************************************/
/*                              FUNCION PERDEDOR                           */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Dibuja la pantalla que aparece cuando se pierde el juego es decir       */
/* cuando explota una mina.                                                */
/***************************************************************************/
void perdedor(void)
{
 /* Variables para dibujar las caritas malas en una posicion de la matriz*/
 //int i,j;
 /* esquina superior izquierda del boton perdedor */
 int x1=mitx-50;
 int y1=mity-30;
 /* esquina inferior derecha del boton perdedor */
 int x2=mitx+50;
 int y2=mity+20;
 explotarminas();
 setlinestyle(0,0,1);
 /* fondo del boton */
 setfillstyle(1,LIGHTGRAY);
 bar(x1,y1,x2,y2);
 /* borde externo */
 setcolor(WHITE);
 line(x1-1,y1-1,x2,y1-1);
 line(x1-1,y1-1,x1-1,y2);
 setcolor(BLACK);
 line(x1-1,y2+1,x2+1,y2+1);
 line(x2+1,y1-1,x2+1,y2+1);
 /* borde interno */
 setcolor(BLACK);
 line(x1+2,y1+2,x2-2,y1+2);
 line(x1+2,y1+2,x1+2,y2-2);
 setcolor(WHITE);
 line(x1+2,y2-2,x2-2,y2-2);
 line(x2-2,y2-2,x2-2,y1+2);
 /* texto del boton */
 settextjustify(CENTER_TEXT,CENTER_TEXT);
 settextstyle(0,0,1);
 setcolor(WHITE);outtextxy(mitx+1,mity-3,"GAME  OVER");
 setcolor(BLACK);outtextxy(mitx,mity-4,"GAME  OVER");
 settextjustify(LEFT_TEXT,TOP_TEXT);
 /* sonido que se emite cuando explotan las minas */
 int ini,fin;//inicio y fin del ciclo
 for(ini=3;ini<=15;ini++)
  sonido(650-(ini*5),10);
 for(fin=15;fin>=3;fin--)
  sonido(500+(fin*5),10);
 /* espera a que se presione Enter o J minscula */
 do
 {
  opc3=getch();
 }while(opc3!=ENTER && opc3!=JMINUS);
 /* dibuja pantalla y pone minas */
 pantalla();
 ponerminas(minas);
}

/***************************************************************************/
/*                            FUNCION PONERMINAS                           */
/*                                                                         */
/* Se le pasa el numero de minas que se pondran en el tablero, no retorna  */
/* ningun valor.                                                           */
/* Pone un respectivo numero de minas en la matriz, cada mina se va        */
/* asignando a una posicion de la matriz de forma aleatoria; en cada       */
/* posicion donde haya una mina se pone un 1(Uno) en la matriz. En esta    */
/* funcion tambien se asignan los numeros que indicaran las minas que hay  */
/* alrededor de estos; el numero de minas totales que hay en la matriz se  */
/* imprimen en pantalla.                                                    */
/***************************************************************************/
void ponerminas(int num)
{
 /* Variables para recorrer la matriz */
 /* num: numero de minas que habra en la matriz*/
 /* i: Posicion X de la matriz(Columnas) */
 /* j: Posicion Y de la matriz(Filas) */
 int i,j;
 /* x: posicion X de la matriz donde habra una mina */
 /* y: posicion Y de la matriz donde habra una mina */
 /* cont: Variable usada para incrementarla hasta que sea igual al numero */
 /*       de minas(num)                                                   */
 int x,y,cont;
 /* itemp, jtemp: indican los cuadros alrededor de cada posicion de la    */
 /*               matriz para poner el respectivo numero de minas que hay */
 /*               alrededor de esa posicion(itemp=columnas, jtemp=filas)  */
 /* ifin,jfin: Indican el inicio y fin para itemp y jtemp(0,1  2)        */
 int itemp,jtemp;
 int ifin,jfin;
 /* numerominas: Guarda el numero de minas de la matriz en una cadena */
 /*              para mostrarla en pantalla                           */
 char *numerominas;
 randomize();
 for(i=0;i<=tamx-1;i++)
 {
  for(j=0;j<=tamy-1;j++)
  {
   tablero2[i][j]=0;
   tablero_decremento[i][j]=0;
  }
 }
 cont=1;
 /* pone las minas en la matriz hasta que el */
 /* contador sea igual al numero de minas    */
 if(num>0)
 {
  do
  {
   x=random(tamx);
   y=random(tamy);
   if(x!=0 && y!=0) //las columnas y las filas deben ser diferentes de cero
   {
	if(x!=tamx-1 && y!=tamy-1)//las columnas y las filas deben ser diferentes
	{                         //al numero maximo de ellas
	 if(tablero2[x][y]!=1) //si no hay mina es diferente de uno(1)
	 {
	  tablero2[x][y]=1; //pone una mina
	  cont=cont+1; //aumenta el contador de las minas
	 }
	}
   }
  }while(cont<=num && cont<=279);
 }

 itemp=0;
 jtemp=0;
 ifin=0;
 jfin=0;
 /* pone los numeros que indican cuantas minas hay alrededor */
 for(i=1;i<=tamx-2;i++)
 {
  for(j=1;j<=tamy-2;j++)
  {
   if(tablero2[i][j]==1)//comprueba si hay una mina
   {
	itemp=i-1;
	jtemp=j-1;
	/* comprueba los cuadros alrededor de la mina */
	for(jfin=0;jfin<3;jfin++)
	{
	 for(ifin=0;ifin<3;ifin++)
	 {
	  if(itemp==i && jtemp==j)
	  {
	   tablero2[itemp][jtemp]=1;
	  }
	  else
	  { /* pone los numeros alrededor de la mina */
	   switch(tablero2[itemp][jtemp])
	   {
		case 0:
			   tablero2[itemp][jtemp]=11;
			   break;
		case 11:
			   tablero2[itemp][jtemp]=12;
			   break;
		case 12:
			   tablero2[itemp][jtemp]=13;
			   break;
		case 13:
			   tablero2[itemp][jtemp]=14;
			   break;
		case 14:
			   tablero2[itemp][jtemp]=15;
			   break;
		case 15:
			   tablero2[itemp][jtemp]=16;
			   break;
		case 16:
			   tablero2[itemp][jtemp]=17;
			   break;
		case 17:
			   tablero2[itemp][jtemp]=18;
			   break;
		case 1:// si hay una mina, queda igual
			   tablero2[itemp][jtemp]=1;
			   break;
	   }
	  }
	  itemp=itemp+1;
	 }//fin para ifin
	 jtemp=jtemp+1;
	 itemp=i-1;
	}//fin para jfin
   }//fin si hay mina
  }//fin para j
 }//fin para i
 xp=1; //situa la posicion X de la matriz en 1
 yp=1; //situa la posicion Y de la matriz en 1
 /* imprime en pantalla el numero de minas */
 numerominas=fcvt(num,0,&dec,&sig);
 setcolor(WHITE);
 outtextxy(90,maxy-10,numerominas);
}

/***************************************************************************/
/*                           FUNCION EXPLOTARMINAS                         */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Dibuja todas las minas en pantalla, para esto se comprueban los valores */
/* en donde la matriz tiene unos.                                          */
/***************************************************************************/
void explotarminas(void)
{
 /* variables para recorrer la matriz en busca de la minas */
 int x,y;
 for(x=1;x<=tamx-2;x++)
 {
  for(y=1;y<=tamy-2;y++)
  {
   if(tablero2[x][y]==1)
   {
	xg=(x*30)+5;
	yg=(y*30)+25;
	setfillstyle(1,RED);
	bar(xg-14,yg-16,xg+14,yg+12);
	mina(xg,yg);
   }
  }
 }
}

/***************************************************************************/
/*                          FUNCION MOSTRARNUMEROS                         */
/*                                                                         */
/* Se le pasa el numero como una cadena de caracteres (El numero debe ir   */
/* entre comillas dobles, Ej. "7"),el color del numero y el color de fondo */
/* del mismo, no retorna ningun valor.                                     */
/* Dibuja los valores que indican el respectivo numero de minas que se     */
/* encuentran alrededor de l.                                             */
/***************************************************************************/
void mostrarnumeros(char *num2,int colornum,int colorfondo)
{
 /* num2: cadena para mostrar el numero de la posicion actual */
 /* colornum: color del numero a mostrar de la posicion actual */
 /* colorfondo: color del fondo del numero a mostrar */
 /* fondo de la posicion actual de juego */
 setfillstyle(1,colorfondo);
 bar(xg-14,yg-16,xg+14,yg+12);
 /* muestra el numero de la posicion actual */
 settextjustify(CENTER_TEXT,CENTER_TEXT);
 settextstyle(0,0,1);
 setcolor(colornum);
 if(atoi(num2)==0)
  outtextxy(xg,yg-2," ");
 else
  outtextxy(xg,yg-2,num2);
}

/***************************************************************************/
/*                              FUNCION SINMINA                            */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Comprueba cada posicion de la matriz y muestra el valor que indica el   */
/* numero de minas que se encuentran alrededor de l. Ademas calcula la    */
/* puntuacion que se va obteniendo y la muestra en pantalla.               */
/***************************************************************************/
void sinmina(void)
{
 /* comprueba si esta situado dentro de la matriz*/
 if(xp>=1 && xp<=tamx-2 && yp>=1 && yp<=tamy-2)
 {
  /* convierte posicion de la matriz a posicion grafica de pantalla */
  xg=(xp*30)+5;
  yg=(yp*30)+25;
  /* busca que numero esta en la posicion actual de la matriz */
  if(tablero2[xp][yp]!=1)
  {
   switch(tablero2[xp][yp])
   {
	case 0:
		   mostrarnumeros("0",WHITE,BLUE);
		   break;
	case 11:
		   mostrarnumeros("1",LIGHTGRAY,BLUE);
		   break;
	case 12:
		   mostrarnumeros("2",LIGHTGREEN,BLUE);
		   break;
	case 13:
		   mostrarnumeros("3",LIGHTMAGENTA,BLUE);
		   break;
	case 14:
		   mostrarnumeros("4",LIGHTRED,BLUE);
		   break;
	case 15:
		   mostrarnumeros("5",LIGHTCYAN,BLUE);
		   break;
	case 16:
		   mostrarnumeros("6",YELLOW,BLUE);
		   break;
	case 17:
		   mostrarnumeros("7",BROWN,BLUE);
		   break;
	case 18:
		   mostrarnumeros("8",BROWN,BLUE);
		   break;
   }
  }
 }
 /* por cada posicion que se destapa, incrementa el puntaje en 100 */
 if(xp>=1 && xp<=tamx-2 && yp>=1 && yp<=tamy-2)
 {
  puntos=puntos+100;
  if(puntos>=999900)
  {
   puntos=999900;
  }
 }
 /* convierte el puntaje a una cadena de caracteres */
 puntuacion=fcvt(puntos,0,&dec,&sig);
 /* muestra el puntaje en la barra de informacion */
 setcolor(WHITE);
 setfillstyle(1,DARKGRAY);
 bar(mitx+160,maxy-15,mitx+290,maxy-5);
 settextjustify(LEFT_TEXT,CENTER_TEXT);
 outtextxy(mitx+180,maxy-10,puntuacion);

 /* cuenta el numero de espacios iguales a 100 hasta que decremento */
 /* sea igual al numero de minas.                                   */
 int x,y;
 /* pone un 100 en la posicion actual de la matriz tablero_decremento */
 if(xp>=1 && xp<=tamx-2 && yp>=1 && yp<=tamy-2)
 {
  if(tablero2[xp][yp]!=1)
  {
   tablero_decremento[xp][yp]=100;
  }
 }
 decremento=280; // numero de posiciones en la matriz 20x14
 /* disminuye en uno el decremento cada vez que encuentra un 100 en */
 /* la matriz tablero_decremento. */
 for(x=1;x<=tamx-2;x++)
 {
  for(y=1;y<=tamy-2;y++)
  {
   if(tablero_decremento[x][y]==100)
   {
	decremento=decremento-1;
   }
  }
 }

}

/***************************************************************************/
/*                              FUNCION SONIDO                             */
/*                                                                         */
/* Se le pasa la frecuencia y el tiempo de duracion en milisegundos del    */
/* sonido a emitir.(1 segundo = 1000 milisegundos).                        */
/* Emite un sonido de acuerdo a la fracuencia dada.                        */
/***************************************************************************/
void sonido(int frecuencia,long int tiempo)
{
 sound(frecuencia);
 delay(tiempo);
 nosound();
}

/***************************************************************************/
/*                               FUNCION MINA                              */
/*                                                                         */
/* Se le pasa la posicion grafica central donde se desea dibujar la mina,  */
/* no retorna ningun valor.                                                */
/* Dibuja una mina en la posicion grafica central que se le indique.       */
/***************************************************************************/
void mina(int xg,int yg)
{
 yg=yg-2;
 /* puntas de la mina */
 setcolor(BLACK);
 setlinestyle(0,0,3);
 line(xg,yg-7,xg,yg+7);
 line(xg-7,yg,xg+7,yg);
 line(xg-5,yg-5,xg+5,yg+5);
 line(xg+5,yg-5,xg-5,yg+5);
 /* circulo de la mina */
 setlinestyle(0,0,1);
 setfillstyle(1,DARKGRAY);
 fillellipse(xg,yg,5,5);
 /* punto central */
 setlinestyle(0,0,3);
 line(xg-1,yg,xg+1,yg);
}

/***************************************************************************/
/*                           FUNCION MOSTRARCUADRO                         */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Dibuja el cuadro de la posicion actual de juego y ademas borra el de la */
/* posicion anterior.                                                      */
/***************************************************************************/
void mostrarcuadro(void)
{
 /* posicion anterior*/
 xg_ant=(xp_ant*30)+5;
 yg_ant=(yp_ant*30)+25;
 setlinestyle(0,0,1);
 setcolor(BLACK);
 rectangle(xg_ant-15,yg_ant-17,xg_ant+15,yg_ant+13);
 /* posicion actual */
 xg=(xp*30)+5;
 yg=(yp*30)+25;
 setlinestyle(0,0,1);
 setcolor(LIGHTRED);
 rectangle(xg-15,yg-17,xg+15,yg+13);
}

/***************************************************************************/
/*                              FUNCION MENSAJE                            */
/*                                                                         */
/* Se le pasa las posiciones graficas en donde se desea mostrar el mensaje,*/
/* (x,y seran el centro en donde se dibujara el boton), y el texto del     */
/* mensaje a mostrar como una cadena de caracteres (El mensaje debe ir     */
/* entre comillas dobles (Ej."Texto del mensaje"),no retorna ningun valor. */
/* Dibuja un mensaje en pantalla con el texto que se la pasa.              */
/***************************************************************************/
void mensaje(int x,int y,char *mens)
{
 unsigned int sizemensaje;
 void *debajomensaje;
 int largox,largoy;
 largox=(strlen(mens)*8)/2 + 12;
 largoy=16;
 if(strlen(mens)<=70)
 {
  sizemensaje=imagesize(x-(largox+5),y-(largoy+5),x+(largox+5),y+(largoy+5));
  if((debajomensaje=malloc(sizemensaje))==NULL)
  {
   mensaje(mitx,mity,"Error: Memoria insuficiente para mostrar este cuadro de dilogo");
  }
  else
  {
   getimage(x-(largox+5),y-(largoy+5),x+(largox+5),y+(largoy+5),debajomensaje);
   setlinestyle(0,0,1);
   /* fondo del boton */
   setfillstyle(1,LIGHTGRAY);bar(x-largox,y-largoy,x+largox,y+largoy);
   /* borde externo */
   setcolor(WHITE);
   line(x-largox,y-largoy,x-largox,y+largoy);
   line(x-largox,y-largoy,x+largox,y-largoy);
   setcolor(DARKGRAY);
   line(x+largox,y-largoy,x+largox,y+largoy);
   line(x-largox,y+largoy,x+largox,y+largoy);
   /* borde interno */
   setcolor(WHITE);
   rectangle((x-largox)+5,(y-largoy)+5,(x+largox)-4,(y+largoy)-4);
   setcolor(DARKGRAY);
   rectangle((x-largox)+4,(y-largoy)+4,(x+largox)-5,(y+largoy)-5);
   /* texto del mensaje */
   settextjustify(CENTER_TEXT,CENTER_TEXT);
   settextstyle(0,0,1);
   setcolor(WHITE);
   outtextxy(x,y,mens);
   setcolor(BLACK);
   outtextxy(x,y,mens);
   getch();
   putimage(x-(largox+5),y-(largoy+5),debajomensaje,COPY_PUT);
   free(debajomensaje);
  }
 }
 else
 {
  mensaje(mitx,mity,"Error: 70 Caracteres Maximo");
 }
}

/***************************************************************************/
/*                             FUNCION ACERCADE                            */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Dibuja la pantalla acerca de, en donde se encuentra la informacion      */
/* respectiva de los creadores, y la version y el nombre del programa.     */
/***************************************************************************/
void acercade(void)
{
 int x1,y1,x2,y2;
 unsigned int sizeacercade;
 void *debajoacercade;
 x1=mitx-130;
 y1=mity-70;
 x2=mitx+130;
 y2=mity+70;
 sizeacercade=imagesize(x1-5,y1-5,x2+5,y2+5);
 if((debajoacercade=malloc(sizeacercade))==NULL)
 {
  mensaje(mitx,mity,"Error: Memoria insuficiente para mostrar este cuadro de dilogo");
 }
 else
 {
  getimage(x1-5,y1-5,x2+5,y2+5,debajoacercade);
  setlinestyle(0,0,1);
  /* fondo del boton*/
  setfillstyle(1,LIGHTGRAY);
  bar(x1,y1,x2,y2);
  /* fondo del titulo */
  setfillstyle(1,LIGHTGRAY);
  bar(x1+3,y1+3,x2-3,y1+21);
  /* fondo de la informacion */
  setfillstyle(1,LIGHTGRAY);
  bar(x1+3,y1+25,x2-3,y2-3);
  /* borde titulo */
  setcolor(WHITE);
  rectangle(x1+3,y1+3,x2-2,y1+22);
  setcolor(DARKGRAY);
  rectangle(x1+2,y1+2,x2-3,y1+21);
  /* borde externo */
  setcolor(WHITE);
  line(x1-1,y1-1,x2,y1-1);
  line(x1-1,y1-1,x1-1,y2);
  setcolor(DARKGRAY);
  line(x1-1,y2+1,x2+1,y2+1);
  line(x2+1,y1-1,x2+1,y2+1);
  /* borde informacion */
  setcolor(WHITE);
  rectangle(x1+3,y1+26,x2-2,y2-2);
  setcolor(DARKGRAY);
  rectangle(x1+2,y1+25,x2-3,y2-3);
  /* minas de los lados del titulo */
  mina(x1+25,y1+14);
  mina(x2-25,y1+14);
  /* texto del titulo */
  settextjustify(CENTER_TEXT,TOP_TEXT);
  settextstyle(0,0,1);
  setcolor(WHITE);
  outtextxy(mitx+1,y1+10,"B U S C A M I N A S");
  setcolor(BLACK);
  outtextxy(mitx,y1+9,   "B U S C A M I N A S");
  /* texto de la informacion */
  setcolor(BLUE);
  outtextxy(mitx+1,y1+56,  "-----------------------");
  outtextxy(mitx+1,y1+71,  "ALEJANDRO MUOZ ANDRADE");
  outtextxy(mitx+1,y1+86,  "-----------------------");
  setcolor(BLACK);
  outtextxy(mitx,y1+35,  "Copyright @ 2001-2002 por");
  setcolor(WHITE);
  outtextxy(mitx,y1+55,  "-----------------------");
  outtextxy(mitx,y1+70,  "ALEJANDRO MUOZ ANDRADE");
  outtextxy(mitx,y1+85,  "-----------------------");
  setcolor(BLACK);
  outtextxy(mitx,y1+110, "Universidad del Cauca");
  outtextxy(mitx,y1+125, "Popayn - Colombia");
  getch();
  putimage(x1-5,y1-5,debajoacercade,COPY_PUT);
  free(debajoacercade);
 }
}

/***************************************************************************/
/*                              FUNCION AYUDA                              */
/*                                                                         */
/* No se le pasa ningun parametro, ni retorna ningun valor.                */
/* Lee una opcion del teclado e incrementa o decrementa el numero de       */
/* paginas que tiene la ayuda.                                             */
/***************************************************************************/
void ayuda(void)
{
 char opc_ayuda;
 int pag,maximo;
 pag=1;
 maximo=2;
 cleardevice();
 setpalette(CYAN,8);    // cambia el color CYAN a morado oscuro
 setpalette(YELLOW,38); // cambia el color YELLOW a sapote
 paginas(pag);
 /* lee una opcion del teclado hasta presionar la tecla Escape */
 do
 {
  opc_ayuda=getch();
  switch(opc_ayuda)
  {
   case AVPAG://aumenta las paginas
			 if(pag>0 && pag<maximo)
			 {
			  pag++;
			  paginas(pag);
			 }
			 break;
   case REPAG://disminuye las paginas
			 if(pag>1)
			 {
			  pag--;
			  paginas(pag);
			 }
			 break;
   case F5:
			 acercade();
			 break;
   case ESC:
			 break;
   default:
			 break;
  }
 }while(opc_ayuda!=ESC);
 setallpalette(&paleta);
}

/***************************************************************************/
/*                              FUNCION PAGINAS                            */
/*                                                                         */
/* Se le pasa el numero de paginas que tendra la ayuda, no retorna ningun  */
/* valor.                                                                  */
/* Dibuja cada una de las paginas que corresponden a la Ayuda, e imprime   */
/* el texto que se va a mostrar en cada una de las pantallas.              */
/***************************************************************************/
void paginas(int num)
{
 setlinestyle(0,0,1);
 /* fondo de la ayuda 1*/
 setfillstyle(1,CYAN);
 bar(0,0,maxx,maxy);
 /* barra de titulo */
 setfillstyle(1,YELLOW);
 bar(0,0,maxx,20);
 setcolor(DARKGRAY);
 rectangle(2,2,maxx-2,20);
 /* barra de informacion */
 setfillstyle(1,YELLOW);
 bar(0,maxy-20,maxx,maxy);
 setcolor(DARKGRAY);
 rectangle(2,maxy-20,maxx-2,maxy-2);
 /* texto del titulo*/
 settextjustify(LEFT_TEXT,TOP_TEXT);
 setcolor(BLACK);
 outtextxy(10,8,"AYUDA BUSCAMINAS");
 /* texto de la informacion */
 setcolor(BLACK);
 outtextxy(10,maxy-13,"RePg: ANTERIOR      AvPg: SIGUIENTE       F5: ACERCA DE      ESC: REGRESAR");
 /* borde de la pantalla */
 setcolor(WHITE);
 setlinestyle(0,0,3);
 rectangle(0,0,maxx,maxy);
 /* texto de contenido de cada una de las paginas */
 setcolor(LIGHTGRAY);
 settextstyle(0,0,1);
 settextjustify(LEFT_TEXT,CENTER_TEXT);
 switch(num)
 {
  case 1:
		 outtextxy(15,40, "----------------------------------------------------------------------------");
		 outtextxy(15,60, "OBJETIVO DEL JUEGO");
		 outtextxy(15,65, "__________________");
		 outtextxy(15,90, " El juego consiste en encontrar todas las minas que se encuentran escondidas");
		 outtextxy(15,110," en el tablero, para dicho proposito se deben tener en cuenta los numeros   ");
		 outtextxy(15,130," que van apareciendo en cada espacio donde no hay minas, estos indican el   ");
		 outtextxy(15,150," numero de minas que hay alrededor de esa casilla. Si no hay minas alrededor");
		 outtextxy(15,170," de esa casilla, aparecer un espacio en blanco.                            ");
		 outtextxy(15,200," Ejemplo:                                                                   ");
		 outtextxy(15,440,"-------------------------------------- 1 -----------------------------------");
		 /* dibuja el ejemplo de como se situan los numeros y las minas */
		 setfillstyle(1,BLUE);
		 bar(30,230,120,320);
		 setfillstyle(1,RED);
		 bar(90,230,120,260);
		 bar(90,260,120,290);
		 bar(60,290,90,320);
		 mina(105,246);
		 mina(105,276);
		 mina(75,306);
		 setlinestyle(0,0,1);
		 setcolor(BLACK);
		 rectangle(30,230,120,320);
		 line(60,230,60,320);
		 line(90,230,90,320);
		 line(30,260,120,260);
		 line(30,290,120,290);
		 /* muestra los numeros del ejemplo */
		 settextstyle(0,0,1);
		 settextjustify(CENTER_TEXT,CENTER_TEXT);
		 setcolor(LIGHTMAGENTA);outtextxy(75,275,"3");
		 setcolor(LIGHTGREEN);outtextxy(75,245,"2");
		 setcolor(LIGHTGRAY);outtextxy(45,275,"1");
		 setcolor(LIGHTGREEN);outtextxy(105,305,"2");
		 setcolor(LIGHTGRAY);outtextxy(45,305,"1");
		 break;
  case 2:
		 outtextxy(15,40, "----------------------------------------------------------------------------");
		 outtextxy(15,60, "INICIO DEL JUEGO");
		 outtextxy(15,65, "________________");
		 outtextxy(15,90, " * En el menu Juego, elija la opcin Nuevo o presione Enter para iniciarlo. ");
		 outtextxy(15,110," * El area de juego esta formada por el tablero, el marcador de minas que se");
		 outtextxy(15,130,"   encuentran escondidas y el marcador de puntuacin.                       ");
		 outtextxy(15,150," * Para pasar de una casilla a otra utilice las flechas del teclado.        ");
		 outtextxy(15,170," * Para abrir una casilla presione la tecla Supr.                            ");
		 outtextxy(15,190," * En el menu Juego elija la opcin Cancelar si desea regresar al juego que ");
		 outtextxy(15,210,"   est activo.");
		 outtextxy(15,230," * Tenga en cuenta desactivar la tecla de maysculas de su teclado, si sta ");
		 outtextxy(15,250,"   tecla est activada, las opciones del menu no funcionarn.");
		 outtextxy(15,440,"-------------------------------------- 2 -----------------------------------");
		 break;
  default:
		 break;
 }
}
